/*****************************************************************************
 * 
 * based on original ideas of Richard Benson and Brian Tischler 
 *
 *****************************************************************************/

#include "Particle.h"
#include "ParticleSystem.h"


// Update the particle to set its new properties based
// on the time interval since it was last updated. The
// velocity, colour, size, trasparency, position, acceleration
// and angular dispersion are all updated.
int UpdateParticle(Particle *ths,  float fTimeDelta ){
 if ( ths->fAge + fTimeDelta >= ths->fLifetime )  {
  ths->fAge = -1.0f;
  return FALSE;
 }
 else{
  ths->fAge += fTimeDelta;
  ths->vPrevLocation.x = ths->vLocation.x;
  ths->vPrevLocation.y = ths->vLocation.y;
  ths->vPrevLocation.z = ths->vLocation.z;
  ths->vLocation.x += ths->vVelocity.x * fTimeDelta;
  ths->vLocation.y += ths->vVelocity.y * fTimeDelta;
  ths->vLocation.z += ths->vVelocity.z * fTimeDelta;
  if(ths->pParent->bIsColliding)  {
  //if ( IsColliding(ths->pParent) )  {
   if ( ths->vLocation.y < GROUND ) {
    if ( ths->vVelocity.y > -5.0f ){
     ths->vLocation.y = GROUND;
     ths->vVelocity.y = 0.0f;
    }
    else    {
     ths->vLocation.y = ths->vPrevLocation.y; 
     ths->vVelocity.y = -ths->vVelocity.y*ths->pParent->fBoing;
    }
   }
  }
  ths->vVelocity.x += (9.8f * ths->vParticleGravity.x * fTimeDelta );
  ths->vVelocity.y += (9.8f *  ths->vParticleGravity.y * fTimeDelta );
  ths->vVelocity.z += (9.8f *  ths->vParticleGravity.z * fTimeDelta );
  ths->fAlpha += ths->fAlphaDelta * fTimeDelta;
  ths->vColor.r += ths->vColorDelta.r * fTimeDelta;
  ths->vColor.g += ths->vColorDelta.g * fTimeDelta;
  ths->vColor.b += ths->vColorDelta.b * fTimeDelta;
  ths->fSize += ths->fSizeDelta * fTimeDelta;
  ths->vParticleGravity.x += ths->vGravityDelta.x * fTimeDelta;
  ths->vParticleGravity.y += ths->vGravityDelta.y * fTimeDelta;
  ths->vParticleGravity.z += ths->vGravityDelta.z * fTimeDelta;
 }
 return TRUE;
}
