/*****************************************************************************
 * 
 * based on original ideas of Richard Benson and Brian Tischler 
 *
 *****************************************************************************/

#include "Particle.h"
#include "ParticleSystem.h"
#include <math.h>

// Update the particle system at time "fTime", this will decide
// hown many particles to dispatch at this time and what properites to
// give them. It will also tell the active particles to updated themselves.
void UpdateParticleSystem(aParticleSystem* ths, float fTime){
 int i;
 static float fThetaHoriz = 0.0f;
 static float fThetaVert = 180.0f;
 float fTimeDelta = fTime - ths->fTimeLastUpdate;
 float fParticlesNeeded;
 unsigned int uParticlesCreated;
 ths->fTimeLastUpdate = fTime;
 if (ths->bIsMoving) {
  fThetaHoriz += 25.0f * fTimeDelta;
  fThetaVert += 35.0f * fTimeDelta*ths->fVarx;
  if ( fThetaHoriz > 359.0f )   {
    fThetaHoriz = 0.0f;
  }
  if ( fThetaVert > 359.0f )   {
    ths->fVarx = RANDOM_NUM*1.0f+1.0f;
    fThetaVert = 0.0f;
  }
  ths->vLocation.x = 20.0f * (float)cos( fThetaHoriz * 0.01745f );
  ths->vLocation.y =  7.0f * (float)sin( fThetaVert * 0.01745f );
  ths->vLocation.z = 20.0f * (float)sin( fThetaHoriz * 0.01745f )*(float)sin( fThetaVert * 0.01745f );
 }
 else {
  ths->vLocation.x = 0.0f;
  ths->vLocation.y = 0.0f;
  ths->vLocation.z = 0.0f;
  
 }
 ths->uParticlesAlive = 0;
 for ( int i=0; i<MAX_PARTICLES; i++) { // update the currently active particles
  if (ths->rParticles[i].fAge >= 0.0f )
   if (UpdateParticle(&(ths->rParticles[i]),fTimeDelta ))
    ths->uParticlesAlive++;
 }
 // calculate the number of additional partices needed to maintain the emission rate
 ths->uParticlesPerSec=(unsigned int)(ths->fParticlesPerSec);
 fParticlesNeeded = ths->uParticlesPerSec * fTimeDelta + ths->fEmissionResidue;
 uParticlesCreated = (unsigned int)fParticlesNeeded;
 if ( !ths->bIsSuppressed )  {
  ths->fEmissionResidue = fParticlesNeeded - uParticlesCreated;
 } 
 else {  
  ths->fEmissionResidue = 0.0f;  
  uParticlesCreated = 0; 
 }
 if ( uParticlesCreated > 0 ) { // we need to add an additional "uParticlesCreated" particles
  for ( i=0; i<MAX_PARTICLES; i++ )  {
   if ( !uParticlesCreated )break;
   if ( ths->rParticles[i].fAge < 0.0f ){  // re-use a particle that has finished and disappeared
    ths->rParticles[i].fAge = 0.0f;
    ths->rParticles[i].fLifetime = ths->fLifeTime+RANDOM_NUM * ths->fLifeVar;
    Clamp( ths->rParticles[i].fLifetime, MIN_LIFETIME, MAX_LIFETIME ); 
    ths->rParticles[i].fAlpha = ths->fAlphaStart + RANDOM_NUM * ths->fAlphaVar;
    Clamp( ths->rParticles[i].fAlpha, MIN_ALPHA, MAX_ALPHA );
    ths->rParticles[i].fAlphaDelta = (ths->fAlphaEnd - ths->rParticles[i].fAlpha) / ths->rParticles[i].fLifetime;
    ths->rParticles[i].vColor.r = ths->vColorStart.r+ RANDOM_NUM * ths->vColorVar.r;
    ths->rParticles[i].vColor.g = ths->vColorStart.g + RANDOM_NUM * ths->vColorVar.g;
    ths->rParticles[i].vColor.b = ths->vColorStart.b + RANDOM_NUM * ths->vColorVar.b;
    Clamp( ths->rParticles[i].vColor.r, 0.0f, 1.0f );
    Clamp( ths->rParticles[i].vColor.g, 0.0f, 1.0f );
    Clamp( ths->rParticles[i].vColor.b, 0.0f, 1.0f );
    ths->rParticles[i].vColorDelta.r = ((ths->vColorEnd.r+ RANDOM_NUM * ths->vColorVar.r) - ths->rParticles[i].vColor.r) / ths->rParticles[i].fLifetime;
    ths->rParticles[i].vColorDelta.g = ((ths->vColorEnd.g+ RANDOM_NUM * ths->vColorVar.g) - ths->rParticles[i].vColor.g) / ths->rParticles[i].fLifetime;
    ths->rParticles[i].vColorDelta.b = ((ths->vColorEnd.b+ RANDOM_NUM * ths->vColorVar.b) - ths->rParticles[i].vColor.b) / ths->rParticles[i].fLifetime;
    ths->rParticles[i].fSize = ths->fSizeStart + RANDOM_NUM * ths->fSizeVar;
    Clamp( ths->rParticles[i].fSize, MIN_SIZE, MAX_SIZE );
    ths->rParticles[i].fSizeDelta = (ths->fSizeEnd - ths->rParticles[i].fSize) / ths->rParticles[i].fLifetime;
    ths->rParticles[i].vParticleGravity.x =  ths->vGravityStart.x * GRAVITY + RANDOM_NUM * ths->fGravityVar*GRAVITY;
    Clamp( ths->rParticles[i].vParticleGravity.x, MIN_GRAVITY*GRAVITY, MAX_GRAVITY*GRAVITY );
    ths->rParticles[i].vGravityDelta.x = 
         ( ths->vGravityEnd.x*GRAVITY - ths->rParticles[i].vParticleGravity.x ) / ths->rParticles[i].fLifetime;
    ths->rParticles[i].vParticleGravity.y =  ths->vGravityStart.y * GRAVITY + RANDOM_NUM * ths->fGravityVar*GRAVITY;
    Clamp( ths->rParticles[i].vParticleGravity.y, MIN_GRAVITY*GRAVITY, MAX_GRAVITY*GRAVITY );
    ths->rParticles[i].vGravityDelta.y = 
         ( ths->vGravityEnd.y*GRAVITY - ths->rParticles[i].vParticleGravity.y ) / ths->rParticles[i].fLifetime;
    ths->rParticles[i].vParticleGravity.z =  ths->vGravityStart.z * GRAVITY + RANDOM_NUM * ths->fGravityVar*GRAVITY;
    Clamp( ths->rParticles[i].vParticleGravity.z, MIN_GRAVITY*GRAVITY, MAX_GRAVITY*GRAVITY );
    ths->rParticles[i].vGravityDelta.z = 
         ( ths->vGravityEnd.z*GRAVITY - ths->rParticles[i].vParticleGravity.z ) / ths->rParticles[i].fLifetime;
    ths->vTempVelocity.x = (ths->vLocation.x - ths->vPrevLocation.x)/fTimeDelta;
    ths->vTempVelocity.y = (ths->vLocation.y - ths->vPrevLocation.y)/fTimeDelta;
    ths->vTempVelocity.z = (ths->vLocation.z - ths->vPrevLocation.z)/fTimeDelta;
    ths->rParticles[i].vLocation.x = ths->vPrevLocation.x;
    ths->rParticles[i].vLocation.y = ths->vPrevLocation.y;
    ths->rParticles[i].vLocation.z = ths->vPrevLocation.z;
    ths->rParticles[i].vPrevLocation.x = ths->rParticles[i].vLocation.x;
    ths->rParticles[i].vPrevLocation.y = ths->rParticles[i].vLocation.y;
    ths->rParticles[i].vPrevLocation.z = ths->rParticles[i].vLocation.z;
    float RandomYaw = RANDOM_NUM * 3.14159f * 2.0f;
    float RandomPitch = RANDOM_NUM * ths->fTheta * 3.14159f ;/// 180.0f ;
    ths->rParticles[i].vVelocity.y = (float)(cos( RandomPitch ));
    ths->rParticles[i].vVelocity.x = (float)(sin(RandomPitch) * cos(RandomYaw));
    ths->rParticles[i].vVelocity.z = (float)(sin(RandomPitch) * sin(RandomYaw));
    ths->rParticles[i].vVelocity.x *= ths->fSpeed ;
    ths->rParticles[i].vVelocity.y *= ths->fSpeed ;
    ths->rParticles[i].vVelocity.z *= ths->fSpeed ;
    ths->rParticles[i].pParent=ths;
    uParticlesCreated--;
   }
  }
 }
 ths->vPrevLocation = ths->vLocation; 
}

void DrawParticleSystemParticles(aParticleSystem* ths, float x_Rot,float y_Rot){
int loop;
 for (loop = 0; loop < MAX_PARTICLES; loop++) {
  if (ths->rParticles[loop].fAge >= 0.0f)  {
   glColor4f(ths->rParticles[loop].vColor.r,ths->rParticles[loop].vColor.g,
        ths->rParticles[loop].vColor.b,ths->rParticles[loop].fAlpha);
   glPushMatrix();
   glTranslatef(ths->rParticles[loop].vLocation.x,
           ths->rParticles[loop].vLocation.y,
            ths->rParticles[loop].vLocation.z);
   glRotatef(y_Rot,0.0f,1.0f,0.0f);
   glRotatef(x_Rot,1.0f,0.0f,0.0f);

   glBegin(GL_TRIANGLE_STRIP);      // Build Quad From A Triangle Strip
   glTexCoord2f(1.0f, 1.0f); glVertex3f( ths->rParticles[loop].fSize, ths->rParticles[loop].fSize, 0.0f); // Top Right
   glTexCoord2f(0.0f, 1.0f); glVertex3f(-ths->rParticles[loop].fSize, ths->rParticles[loop].fSize, 0.0f); // Top Left
   glTexCoord2f(1.0f, 0.0f); glVertex3f( ths->rParticles[loop].fSize,-ths->rParticles[loop].fSize, 0.0f); // Bottom Right
   glTexCoord2f(0.0f, 0.0f); glVertex3f(-ths->rParticles[loop].fSize,-ths->rParticles[loop].fSize, 0.0f);// Bottom Left
   glEnd();         
   glPopMatrix();
  }
 }
 if( ths->bIsColliding) { // draw the ground
  glPushAttrib(GL_TEXTURE_BIT);
  glDisable(GL_TEXTURE_2D);
  glLineWidth(3.0f);
  glColor4f(0.4f,0.6f,0.6f,0.9f);
  glBegin(GL_LINES); // this is the emitter
  glVertex3f( ths->vLocation.x,ths->vLocation.y-ths->fSizeStart*0.5f,ths->vLocation.z);
  glVertex3f( ths->vLocation.x,ths->vLocation.y-ths->fSizeStart*3.0f,ths->vLocation.z);
  glEnd();
  glTranslatef(0.0f,GROUND,0.0f);
  glColor4f(0.0f,0.3f,0.3f,0.7f);
  glLineWidth(1.0f);
  glBegin(GL_LINE_STRIP);
  glVertex3f(-20.0f,0.0f,-20.0f);
  glVertex3f(-20.0f,0.0f, 20.0f);
  glVertex3f( 20.0f,0.0f, 20.0f);
  glVertex3f( 20.0f,0.0f,-20.0f);
  glVertex3f(-20.0f,0.0f,-20.0f);
  glEnd();
  glBegin(GL_LINE_STRIP);
  glVertex3f(-10.0f,0.0f,-10.0f);
  glVertex3f(-10.0f,0.0f, 10.0f);
  glVertex3f( 10.0f,0.0f, 10.0f);
  glVertex3f( 10.0f,0.0f,-10.0f);
  glVertex3f(-10.0f,0.0f,-10.0f);
  glEnd();
  glColor4f(0.1f,0.1f,0.1f,1.0f);
  glBegin(GL_QUADS);
  glVertex3f(-20.0f,0.0f,-20.0f);
  glVertex3f(-20.0f,0.0f, 20.0f);
  glVertex3f( 20.0f,0.0f, 20.0f);
  glVertex3f( 20.0f,0.0f,-20.0f);
  glEnd();
  glPopAttrib();
 }
}

