/*****************************************************************************
 * 
 * based on original ideas of Richard Benson and Brian Tischler 
 *
 *****************************************************************************/

#if _WIN32  // windows

#pragma warning( disable : 4244 4305 4267 4101 )
#include <windows.h>
#include "gl\gl.h"   // Header File For The OpenGL32 Library
#include "gl\glu.h"  // Header File For The GLu32 Library
#include "GL\glut.h"
#include "GL\glui.h"

#else  // mac

#include <sys/time.h>
#include <stdlib.h>
#include <opengl/gl.h>
#include <opengl/glu.h>
#include <GLUT/GLUT.h>
#include "gl/glui.h"
#define TRUE 1
#define FALSE 0
#endif

#define MAX_PARTICLES 7500

//A number between 0.0000 and 1.00000
#define RANDOM_NUM ((float)(rand())/(float)(RAND_MAX))

// Some defines for our ParticleSystem.   This makes it easy to change settings
// by using defines and putting them all in one place.
#define MIN_SPEED  0.0f     // in world units / sec
#define MIN_LIFETIME 0.1f   // in seconds
#define MIN_SPREAD  0.01f   // in degrees
#define MIN_EMISSION 1.0f   // in particles / sec
#define MIN_SIZE  0.1f      // in world units
#define MIN_GRAVITY  -0.5f  // as a multiple of normal gravity 
#define MIN_ALPHA  0.0f     // as a fraction (transparency) 

#define MAX_SPEED  25.0f     // in world units / sec
#define MAX_LIFETIME 15.0f   // in seconds
#define MAX_SPREAD  180.0f   // in degrees
#define MAX_EMISSION 1000.0f // in particles / sec
#define MAX_SIZE  4.0f       // in world units
#define MAX_GRAVITY  0.5f    // as a multiple of normal gravity 
#define MAX_ALPHA  1.0f      // as a ratio 
#define GRAVITY 9.8f         // G

// useful macro to guarantee that values are within a given range
#define Clamp(x, min, max)  x = (x<min  ? min : x<max ? x : max);


typedef struct aParticleSystemTAG {
 float fTimeLastUpdate;
 int   bIsColliding;
 int   bIsMoving;
 int   bIsSuppressed;
 int   bIsFloor;
 float fLifeTime;
 float fLifeVar;
 float fAlphaStart;
 float fAlphaEnd;
 float fAlphaVar;
 float fSizeStart;
 float fSizeEnd;
 float fSizeVar;
 float fSpeed;
 float fGravityVar;
 float fSpeedVar;
 float fTheta;
 float fBoing;                       // coefficient of restitution (bouncyness)
 vector vPrevLocation;
 vector vLocation;
 vector vGravityStart;
 vector vGravityEnd;
 colour vColorStart;
 colour vColorEnd;
 colour vColorVar;
 unsigned int uParticlesPerSec;      // this is integer version of next member
 float fParticlesPerSec;             // number of particles to create per unit time  
 unsigned int uParticlesAlive;       // how may particles are active
 Particle rParticles[MAX_PARTICLES]; // list of all possible particles
 float fEmissionResidue;
 vector vTempVelocity;
 float fVarHoriz;
 float fVarz;
 float fVarx;
 int iParticleTexture;               // which texture map is this system using
} aParticleSystem;

void DrawParticleSystemParticles(aParticleSystem*,float,float);
void UpdateParticleSystem(aParticleSystem*,float fTime);

